﻿using MathNet.Numerics.Statistics;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace ConsoleApp1
{
    public class Program
    {
        static void Main(string[] args)
        {
            double[] dblHeights = LoadData("data.txt");

            //double [] dblHeights = (from elem in heightInfo
            //                 select (double)elem).ToArray();

            Console.WriteLine($"# of data: {dblHeights.Length}");
            Console.WriteLine($"Mean: {Statistics.Mean(dblHeights)}"); // 산술평균
            Console.WriteLine($"GeometricMean: {Statistics.GeometricMean(dblHeights)}"); // 기하평균
            Console.WriteLine($"HarmonicMean: {Statistics.HarmonicMean(dblHeights)}"); // 조화평균
            Console.WriteLine($"RootMeanSquare: {Statistics.RootMeanSquare(dblHeights)}"); // 제곱평균

            {
                Histogram hist = new Histogram(dblHeights, 6);
                Console.WriteLine();
                hist.Output();
                Console.WriteLine();
            }

            {
                Histogram hist = new Histogram(dblHeights, 6, 141, 170);
                Console.WriteLine();
                hist.Output();
                Console.WriteLine();
            }

            {
                Histogram hist = GetHistogram(dblHeights);
                Console.WriteLine();
                hist.Output();
                Console.WriteLine($"Mean of histogram: {hist.GetMean()}");
                Console.WriteLine();
            }
        }

        public static Histogram GetHistogram(double [] dblHeights)
        {
            Histogram hist = new Histogram();

            hist.AddBucket(new Bucket(141, 145));
            hist.AddBucket(new Bucket(145, 150));
            hist.AddBucket(new Bucket(150, 155));
            hist.AddBucket(new Bucket(155, 160));
            hist.AddBucket(new Bucket(160, 165));
            hist.AddBucket(new Bucket(165, 170));

            hist.AddData(dblHeights); // LowerBound < x <= UpperBound
                                      // if No LowerBound, decrease LowerBound at 0-index bucket
            return hist;
        }

        public static double[] LoadData(string fileName)
        {
            List<double> list = new List<double>();

            foreach (string txt in File.ReadAllLines(fileName))
            {
                if (double.TryParse(txt, out double result) == true)
                {
                    list.Add(result);
                }
            }

            return list.ToArray();
        }
    }

    public static class BucketExtension
    {
        public static double GetClassMark(this Bucket bucket)
        {
            return Math.Round((bucket.LowerBound + bucket.UpperBound) / 2, MidpointRounding.AwayFromZero);
            // return Math.Round((bucket.LowerBound + bucket.UpperBound) / 2, MidpointRounding.ToEven); // default - Banker's Rounding
        }

        public static double GetRelativeFrequency(this Bucket bucket, Histogram histogram)
        {
            return bucket.Count / histogram.DataCount;
        }
    }

    public static class HistogramExtension
    {
        public static void Output(this Histogram histogram)
        {
            for (int i = 0; i < histogram.BucketCount; i++)
            {
                Console.WriteLine($"{histogram[i].GetClassMark()} == {histogram[i].Count}");
            }
        }

        public static double GetMean(this Histogram histogram)
        {
            double mean = 0.0;

            for (int i = 0; i < histogram.BucketCount; i++)
            {
                Bucket bucket = histogram[i];

                double classMark = bucket.GetClassMark();
                double relativeFrequency = bucket.GetRelativeFrequency(histogram);

                mean += (classMark * relativeFrequency);
            }

            return mean;
        }
    }
}
