﻿using MathNet.Numerics.Statistics;
using OxyPlot;
using OxyPlot.Axes;
using OxyPlot.Series;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using ConsoleApp1;

namespace WindowsFormsApp1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ConsoleApp1.Program pg = new ConsoleApp1.Program();

            double[] dblHeights = ConsoleApp1.Program.LoadData("data.txt");
            Histogram hist = ConsoleApp1.Program.GetHistogram(dblHeights);

            plotView1.Model = GetHistogramModel(hist);
        }

        // ImageComments 확장을 Visual Studio 2017에 설치하면 아래의 comment에 그래프 그림이 나옵니다.
        // https://marketplace.visualstudio.com/items?itemName=MsBishop.ImageComments

        /// <summary>
        /// <image url="$(SolutionDir)\..\plotview_1.png" scale="0.5" />
        /// </summary>
        /// <param name="hist"></param>
        /// <returns></returns>
        // https://stackoverflow.com/questions/43636795/xamarin-forms-oxyplot-for-bar-charts
        public PlotModel GetHistogramModel(Histogram hist)
        {
            PlotModel model = new PlotModel();

            CategoryAxis xaxis = new CategoryAxis();
            xaxis.Position = AxisPosition.Bottom;
            xaxis.MajorGridlineStyle = LineStyle.Solid;
            xaxis.MinorGridlineStyle = LineStyle.Dot;

            for (int i = 0; i < hist.BucketCount; i++)
            {
                xaxis.Labels.Add(hist[i].GetClassMark().ToString());
            }

            LinearAxis yaxis = new LinearAxis();
            yaxis.Position = AxisPosition.Left;
            yaxis.MajorGridlineStyle = LineStyle.Dot;
            xaxis.MinorGridlineStyle = LineStyle.Dot;

            ColumnSeries series = new ColumnSeries();
            series.IsStacked = true;
            series.LabelPlacement = LabelPlacement.Base;
            series.LabelFormatString = "{0}";

            for (int i = 0; i < hist.BucketCount; i++)
            {
                ColumnItem item = new ColumnItem(hist[i].Count);
                series.Items.Add(item);
            }

            model = new PlotModel();
            model.Title = "Histogram";

            model.Axes.Add(xaxis);
            model.Axes.Add(yaxis);
            model.Series.Add(series);

            return model;
        }

        /// <summary>
        /// <image url="$(SolutionDir)\..\plotview_2.png" scale="0.5" />
        /// </summary>
        /// <returns></returns>
        // http://docs.oxyplot.org/en/latest/models/series/BarSeries.html
        public PlotModel SamplePlotModel_HorizontalOneSeriesBar()
        {
            var model = new PlotModel { Title = "Cake Type Popularity" };

            //generate a random percentage distribution between the 5
            //cake-types (see axis below)
            var rand = new Random();
            double[] cakePopularity = new double[5];
            for (int i = 0; i < 5; ++i)
            {
                cakePopularity[i] = rand.NextDouble();
            }
            var sum = cakePopularity.Sum();

            var barSeries = new BarSeries
            {
                ItemsSource = new List<BarItem>(new[]
                {
                    new BarItem{ Value = (cakePopularity[0] / sum * 100) },
                    new BarItem{ Value = (cakePopularity[1] / sum * 100) },
                    new BarItem{ Value = (cakePopularity[2] / sum * 100) },
                    new BarItem{ Value = (cakePopularity[3] / sum * 100) },
                    new BarItem{ Value = (cakePopularity[4] / sum * 100) }
                }),

                LabelPlacement = LabelPlacement.Inside,
                LabelFormatString = "{0:.00}%",
            };
            model.Series.Add(barSeries);

            model.Axes.Add(new CategoryAxis
            {
                Position = AxisPosition.Left,
                Key = "CakeAxis",
                ItemsSource = new[]
                    {
                        "Apple cake",
                        "Baumkuchen",
                        "Bundt Cake",
                        "Chocolate cake",
                        "Carrot cake"
                    }
            });

            return model;
        }

        /// <summary>
        /// <image url="$(SolutionDir)\..\plotview_3.png" scale="0.5" />
        /// </summary>
        /// <returns></returns>
        // https://stackoverflow.com/questions/43636795/xamarin-forms-oxyplot-for-bar-charts
        public PlotModel SamplePlotModel_VerticalTwoSeriesBar()
        {
            PlotModel model = new PlotModel();

            CategoryAxis xaxis = new CategoryAxis();
            xaxis.Position = AxisPosition.Bottom;
            xaxis.MajorGridlineStyle = LineStyle.Solid;
            xaxis.MinorGridlineStyle = LineStyle.Dot;
            xaxis.Labels.Add("Mon, 4/24");
            xaxis.Labels.Add("Tue, 4/25");
            xaxis.Labels.Add("Wed, 4/26");
            xaxis.Labels.Add("Thu, 4/27");

            LinearAxis yaxis = new LinearAxis();
            yaxis.Position = AxisPosition.Left;
            yaxis.MajorGridlineStyle = LineStyle.Dot;
            xaxis.MinorGridlineStyle = LineStyle.Dot;

            ColumnSeries s1 = new ColumnSeries();
            s1.IsStacked = true;
            s1.Items.Add(new ColumnItem(20));
            s1.Items.Add(new ColumnItem(60));
            s1.Items.Add(new ColumnItem(40));
            s1.Items.Add(new ColumnItem(50));

            ColumnSeries s2 = new ColumnSeries();
            s2.IsStacked = true;
            s2.Items.Add(new ColumnItem(50));
            s2.Items.Add(new ColumnItem(30));
            s2.Items.Add(new ColumnItem(10));
            s2.Items.Add(new ColumnItem(20));

            model = new PlotModel();
            model.Title = "Xamarin Oxyplot Sample";
            model.Background = OxyColors.Gray;

            model.Axes.Add(xaxis);
            model.Axes.Add(yaxis);
            model.Series.Add(s1);
            model.Series.Add(s2);

            return model;
        }
    }
}
